/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.column;

import org.opengion.hayabusa.db.AbstractEditor;
import org.opengion.hayabusa.db.CellEditor;
import org.opengion.hayabusa.db.DBColumn;
import org.opengion.fukurou.util.XHTMLTag;
import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.fukurou.util.Attributes;
import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.TagBuffer;

/**
 * QRCODE エディターは、jsQR.js を利用した、カメラ映像からQRコードを読み込むクラスです。
 * 読み込んだ文字列は テキストエリアに書き出します。
 *
 * 基本的な構造は、カメラ映像描画開始ボタン、映像表示領域(video)、textarea で構成されます。
 * 映像表示領域(video)に、有効なQRコードが表示されると、スキャンを停止して、テキストエリアに
 * 文字列を表示します。現在は、5秒後に、再スキャンを始めます。その際に、なにかQRコードが見つかると
 * 新しいコードに書き換えます。
 *
 *    &lt;button type='button' id='vidStart' onClick='videoStart();qrScan();'&gt;Video Start&lt;/button&gt;
 *    &lt;video id='player' autoplay style='background-color: black;'&gt;&lt;/video&gt;
 *    &lt;textarea name='≪ｶﾗﾑ名≫' id='outdata' rows='10'cols='80'&gt; &lt;/textarea&gt;
 *
 * script は、取り合えず、CDNサービス を使う事にします。
 *
 * &lt;script src="https://cdn.jsdelivr.net/npm/jsqr@latest/dist/jsQR.min.js"&gt;&lt;!-- --&gt;&lt;/script&gt;
 * &lt;script src="{@SYS.JSP}/option/videocamera.js"&gt;&lt;!-- --&gt;&lt;/script&gt;
 *
 * を使用するページに設定します。(当面は、各画面に、必要都度、記述してください。)
 *
 * @og.rev 7.4.2.1 (2021/05/21) 新規作成
 * @og.group データ編集
 *
 * @version  7.4
 * @author   Kazuhiko Hasegawa
 * @since    JDK11.0,
 */
public class Editor_QRCODE extends AbstractEditor {
	/** このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "7.4.2.1 (2021/05/21)" ;

	private static final String BASE_HTML = "<button type='button' id='vidStart' onClick='videoStart();qrScan();'>Video Start</button><br />"
									+ CR +	"<video id='player' autoplay style='background-color: black;'></video>" ;

	/** 列1 */ protected  String	cols1	;
	/** 列2 */ protected  String	cols2	;
	/** 行1 */ protected  String	rows1	;
	/** 行2 */ protected  String	rows2	;

	/**
	 * デフォルトコンストラクター。
	 * このコンストラクターで、基本オブジェクトを作成します。
	 *
	 * @og.rev 7.4.2.1 (2021/05/21) 新規作成
	 *
	 */
	public Editor_QRCODE() { super(); }		// これも、自動的に呼ばれるが、空のメソッドを作成すると警告されるので、明示的にしておきます。

	/**
	 * コンストラクター。
	 *
	 * @og.rev 7.4.2.1 (2021/05/21) 新規作成
	 *
	 * @param	clm	DBColumnオブジェクト
	 */
	protected Editor_QRCODE( final DBColumn clm ) {
		super( clm );
		final String  disabled = clm.isWritable() ? null : "disabled" ;

		final int r1 = clm.getTotalSize()/Integer.parseInt(size1) + 1;
		rows1 = String.valueOf( r1 );

		final int r2 = clm.getTotalSize()/Integer.parseInt(size2) + 1;
		rows2 = String.valueOf( r2 );

		// size に、"rows,cols" を指定できるように変更
		final String param = StringUtil.nval( clm.getEditorParam(),clm.getViewLength() );
		if( param != null && param.length() != 0 ) {
			final int st = param.indexOf( ',' );
			if( st > 0 ) {
				rows1 = param.substring( 0,st );
				rows2 = rows1 ;
				cols1 = param.substring( st+1 );
				cols2 = cols1;
			}
		}

		// size1,size2 を使わずに、cols1,cols2 を使用。
		if( cols1 == null || cols2 == null ) {
			cols1	= size1  ;
			cols2	= size2	 ;
		}

		// Attributesの連結記述
		attributes = new Attributes()
					.set( "disabled"	, disabled )
					.set( clm.getEditorAttributes() )				// #addAttributes( Attributes ) の代替え
					.add( "class"		, clm.getDbType() );

		tagBuffer.add( XHTMLTag.textareaAttri( attributes ) );
	}

	/**
	 * 各オブジェクトから自分のインスタンスを返します。
	 * 自分自身をキャッシュするのか、新たに作成するのかは、各サブクラスの実装に
	 * まかされます。
	 *
	 * @og.rev 7.4.2.1 (2021/05/21) 新規作成
	 *
	 * @param	clm	DBColumnオブジェクト
	 *
	 * @return	CellEditorオブジェクト
	 * @og.rtnNotNull
	 */
	public CellEditor newInstance( final DBColumn clm ) {
		return new Editor_QRCODE( clm );
	}

	/**
	 * データの編集用文字列を返します。
	 *
	 * @og.rev 7.4.2.1 (2021/05/21) 新規作成
	 *
	 * @param   value 入力値
	 *
	 * @return  データの編集用文字列
	 * @og.rtnNotNull
	 */
	@Override
	public String getValue( final String value ) {
		// TagBufferの連結記述
		return BASE_HTML
				+	new TagBuffer( "textarea" )
						.add( "name"	, name )
						.add( "id"		, "outdata" )				// ID 固定です。
						.add( "cols"	, cols2 )
						.add( "rows"	, rows2 )
						.add( tagBuffer.makeTag() )
						.addBody( value )
						.makeTag();
	}

	/**
	 * name属性を変えた、データ表示/編集用のHTML文字列を作成します。
	 * テーブル上の name に 行番号を付加して、名前_行番号 で登録するキーを作成し,
	 * リクエスト情報を１つ毎のフィールドで処理できます。
	 *
	 * @og.rev 7.4.2.1 (2021/05/21) 新規作成
	 *
	 * @param   row   行番号
	 * @param   value 入力値
	 *
	 * @return  データ表示/編集用の文字列
	 * @og.rtnNotNull
	 */
	@Override
	public String getValue( final int row,final String value ) {
		final String newName = name + HybsSystem.JOINT_STRING + row;

		// TagBufferの連結記述
		return BASE_HTML
				+	new TagBuffer( "textarea" )
						.add( "name"	, newName )
						.add( "id"		, "outdata" )				// ID 固定です。
						.add( "cols"	, cols2 )
						.add( "rows"	, rows2 )
						.add( tagBuffer.makeTag() )
						.addBody( value )
						.makeTag( row,value );
	}
}
